/* Simulation of population dynamics in a neighborhood.
   Inspired by The Atlantic Monthly, "Seeing around Corners",
   http://www.theatlantic.com/issues/2002/04/rauch.htm
   and the research of Thomas Schelling.
   Copyright 2002, Akkana Peck, akkana@shallowsky.com.
   You may use, redistribute or modify this code
   under the terms of the GPL.
 */

#include "grid.h"
#include <stdio.h>

void PopulateGrid(int vacantPer, int bluePer)
{
    int i, j;
    int vac = 0;
    int red=0;
    int blue=0;
    srandom(getpid());
    for (i=0; i < GridX; ++i)
        for (j=0; j < GridY; ++j)
        {
            if ((random() % 100) < vacantPer)
            {
                ++vac;
                Grid[i][j] = 0;
            }
            else if ((random() % 100) < bluePer)
            {
                ++blue;
                Grid[i][j] = 1;
            }
            else
            {
                ++red;
                Grid[i][j] = 2;
            }
        }
    if (Verbose)
        printf("%d vacant, %d blue, %d red\n", vac, blue, red);
}

int Happy(int nsc)
{
    extern int HappyNum;

    // Simple initial happiness: we're happy if we have at
    // least 3 neighbors (out of 8) of the same color.
    if (nsc >= HappyNum)
        return 1;
    return 0;
}

// No diagonals
int CountNeighborsSameColor(int i, int j)
{
    int nsc = 0;
    if (i > 0 && Grid[i-1][j] == Grid[i][j])
        ++nsc;
    if (j > 0 && Grid[i][j-1] == Grid[i][j])
        ++nsc;
    if (i < GridX-1 && Grid[i][j] == Grid[i+1][j])
        ++nsc;
    if (j < GridY-1 && Grid[i][j] == Grid[i][j+1])
        ++nsc;
    return nsc;
}

// Includes diagonals
int CountNeighborsSameColorDiag(int i, int j)
{
    int nsc;
    int ii;
    for (ii = i-1; ii <= i+1; ++ii)
    {
        int jj;
        for (jj = j-1; jj <= j+1; ++jj)
        {
            if (i == ii && j == jj)  // ourselves
                continue;
            else if (Grid[ii][jj] == Grid[i][j])
                ++nsc;
        }
    }
    return nsc;
}

void Move(int i, int j)
{
    // Look for a place to move to where we'd be happier
    int ii;
    for (ii = random() % GridX; ii < GridX; ++ii)
    {
        int jj;
        for (jj = random() % GridY; jj < GridY; ++jj)
        {
            int nsc;
            if (Grid[ii][jj] != 0)  // already occupied
                continue;
            nsc = CountNeighborsSameColor(ii, jj);
            if (Happy(nsc))
            {
                int me = Grid[i][j];
                Grid[i][j] = 0;
                Grid[ii][jj] = me;
                //printf("Moving (%d, %d) to (%d, %d)\n", i, j, ii, jj);
                return;
            }
        }
    }
}

int Step()
{
    int i;
    int happiness = 0;
    int changed = 0;
    for (i=0; i < GridX; ++i)
    {
        int j;
        for (j=0; j < GridY; ++j)
        {
            // Count neighbors of same color
            int nsc = CountNeighborsSameColor(i, j);

            if (Happy(nsc))
                ++happiness;

            // Else, we might try to move
            else if (random() % 6 == 0) {
                Move(i, j);
                ++changed;
            }
        }
    }
    if (Verbose)
        printf("happiness: %4d%%\n", happiness * 100 / GridX / GridY);
    return changed;
}





